//+----------------------------------------------------------------------+
//|                                             WalkForwardOptimizer.mqh |
//|                                   Copyright (c) 2016-2023, Marketeer |
//|                              https://www.mql5.com/en/users/marketeer |
//| MT5 v.1.12              https://www.mql5.com/en/market/product/23068 |
//+----------------------------------------------------------------------+

#define DAYS_PER_WEEK    7
#define DAYS_PER_MONTH   30
#define DAYS_PER_QUARTER (DAYS_PER_MONTH*3)
#define DAYS_PER_HALF    (DAYS_PER_MONTH*6)
#define DAYS_PER_YEAR    (DAYS_PER_MONTH*12)

#define SEC_PER_DAY     (60*60*24)
#define SEC_PER_WEEK    (SEC_PER_DAY*DAYS_PER_WEEK)
#define SEC_PER_MONTH   (SEC_PER_DAY*DAYS_PER_MONTH)
#define SEC_PER_QUARTER (SEC_PER_MONTH*3)
#define SEC_PER_HALF    (SEC_PER_MONTH*6)
#define SEC_PER_YEAR    (SEC_PER_MONTH*12)

#define CUSTOM_DAYS     -1

// flags working in agent contex
#define WFO_FLAG_AUTO_CLOSE_SPLIT_TRADES 1
#define WFO_FLAG_MISMATCH_FIX_OFF        2
#define WFO_FLAG_STOP_ON_RANGE_VIOLATION 4
#define WFO_FLAG_CHECK_TICK_MODELLING    8
// flags working in terminal context
#define WFO_FLAG_STOP_ON_NONSTD_PARAMS  16
#define WFO_FLAG_RESUME_OPTIMIZATION    32

/**
 *
 * Custom Performance Meter support starts here
 *
 */

/**
 * Built-in enum with indices for WFO stats container.
 * All is calculated for in-sample range of dates.
 */
enum WFO_STATS_MAP
{
  WFO_STATS_NP,      // net profit
  WFO_STATS_GP,      // gross profit
  WFO_STATS_GL,      // gross loss
  WFO_STATS_PF,      // profit factor
  WFO_STATS_SHARPE,  // sharpe
  WFO_STATS_TRADES,  // N of trades
  WFO_STATS_DDMAX,   // max drawdown value
  WFO_STATS_DDREL,   // drawdown percent
  WFO_STATS_WIN,     // N of winning trades
  WFO_STATS_LOSS,    // N of losing trades
  WFO_STATS_DEPOSIT, // start deposit
  WFO_STATS_AR,      // average return (% of current balance)
  WFO_STATS_STDEV,   // standard deviation of balance curve
  WFO_STATS_MAP_SIZE
};

/**
 * Built-in function pointer type.
 * EA can implement such function and pass it to wfo_setCustomPerformanceMeter.
 * Then WFO will call it from OnTester with the dates and map filled with actual data.
 * The function should return a custom value for OnTester (TesterStatistics can not be used!)
 */
typedef double (*FUNCPTR_WFO_CUSTOM)(const datetime startDate, const datetime splitDate, const double &map[/*WFO_STATS_MAP*/]);


enum WFO_TIME_PERIOD
{
  none = 0,
  year = DAYS_PER_YEAR,
  halfyear = DAYS_PER_HALF,
  quarter = DAYS_PER_QUARTER,
  month = DAYS_PER_MONTH,
  week = DAYS_PER_WEEK,
  day = 1,
  custom = CUSTOM_DAYS
};

enum WFO_ESTIMATION_METHOD
{
  wfo_built_in_loose,
  wfo_built_in_strict,
  wfo_profit,
  wfo_sharpe,
  wfo_pf,
  wfo_drawdown, // wfo_drawdown (inverted relative DD, 100 - DD%)
  wfo_profit_by_drawdown,
  wfo_profit_trades_by_drawdown,
  wfo_average,
  wfo_expression
};

#import "WalkForwardOptimizer MT5.ex5"
void wfo_setEstimationMethod(WFO_ESTIMATION_METHOD estimation, string formula);
void wfo_setPFmax(double max);
void wfo_setCloseTradesOnSeparationLine(bool b);
void wfo_OnTesterPass();
int wfo_OnInit(WFO_TIME_PERIOD optimizeOn, WFO_TIME_PERIOD optimizeStep, int optimizeStepOffset, int optimizeCustomW, int optimizeCustomS);
int wfo_OnTick();
double wfo_OnTester();
void wfo_OnTesterInit(string optimizeLog);
void wfo_OnTesterDeinit();
void wfo_setCustomPerformanceMeter(FUNCPTR_WFO_CUSTOM funcptr);
void wfo_setAdvancedOptions(const ulong flags);
#import

sinput WFO_TIME_PERIOD wfo_windowSize = year;
 input int wfo_customWindowSizeDays = 0;
sinput WFO_TIME_PERIOD wfo_stepSize = quarter;
 input int wfo_customStepSizePercent = 0; // wfo_customStepSize (%% if > 0 | days if < 0)
 input int wfo_stepOffset = 0;
sinput string wfo_outputFile = "";
sinput WFO_ESTIMATION_METHOD wfo_estimation = wfo_built_in_loose;
sinput string wfo_formula = "";
